
%Simulation parameters: we considered 10 different ground choice model with
%50 and 100 customer types and for each we generated 10 data sets
numTrials=10;
numDataSets=10;
numRevs=100;
numTypesList=[50,100];
%Various percentages of the 5000 data points that we train on.  Also test
%on 2500.
trainPercentage=[0.4, 0.7, 1.0];
%Matrices to store results for the arrivals in every period
resultsMatrix=zeros(numTrials,numDataSets,5*length(trainPercentage));
resultsMatrixRevCounts=zeros(numTrials,numDataSets,2*length(trainPercentage));

for numTypesIndex=1:2
    numTypes=numTypesList(numTypesIndex);
    for trial=1:numTrials
        %Read in true customer types
        customerTypes = dlmread(sprintf('../Generated_Data/NonParamModel/NonParametric%d-%d.txt',trial,numTypes), '%d');
        for d=1:numDataSets
            %Read in sales data
            offer = dlmread(sprintf('../Generated_Data/Sales_Data/SalesData%d-%d-%d.txt',trial,d,numTypes), '%d');
            purchase = dlmread(sprintf('../Generated_Data/Purchases/Purchases%d-%d-%d.txt',trial,d,numTypes), '%d');

            D=size(offer);
            numProds=D(2);

            T=D(1);
            revMatrix=zeros(numRevs,numProds);
            revOpt=0;
            for p=1:length(trainPercentage)
                
                %Break up train and test
                percent=trainPercentage(p);
                trainCutOff=floor(percent*(T/2));
                S=offer(1:trainCutOff,:);
                Z=purchase(1:trainCutOff,:);
                SVal=offer((T/2)+1:T,:);
                ZVal=purchase((T/2)+1:T,:);
                
                %initial guess of preference weights
                v_hat = zeros(1,numProds);
                %Create and maximize log likelihood for MNL
                minus_log_L = @(v) -( sum(sum(Z.*repmat([0 v(2:numProds)], trainCutOff,1),2))  - (sum(log(sum(exp(repmat([0 v(2:numProds)], trainCutOff,1)).*S,2)))) );
                minus_log_LVal = @(v) -( sum(sum(ZVal.*repmat([0 v(2:numProds)],T/2,1),2))  - (sum(log(sum(exp(repmat([0 v(2:numProds)],T/2,1)).*SVal,2)))) );
                [v,fvalMNL] = fmincon(minus_log_L,v_hat,[],[]);


  
                %Exponentiate to get preference weights
                prefWeights= exp(v);

                %Get arrival probs for MC choice model
                lambda = prefWeights/sum(prefWeights);

                %Get transition probs for MC model
                P=zeros(numProds,numProds);

                for i=1:numProds

                    for j=1:numProds

                      if i ~= j
                        P(i,j)= lambda(j)/(1-lambda(i));   
                      end
                   end

                end

                %Create function for MC likelihood
                f = @(params)MCLikelihoodParams( S, Z, params, numProds, trainCutOff );

                %constraint saying that all rows of transition matrix have 
                %to sum to less than 1 and same for arrival probs
                Aone=zeros(numProds,numProds^2);
                identity=eye(numProds);

                for i=1:numProds
                    insert=repmat(identity(i,:),1,numProds);
                    Aone(i,:)=insert;
                end

                AoneAdd=zeros(1,numProds^2);

                AoneFinal=vertcat(AoneAdd,Aone);

                Atwo=zeros(numProds+1,numProds);

                Atwo(1,:)=ones(1,numProds);

                A=horzcat(Atwo,AoneFinal);

                b=ones(numProds+1,1);

                lb=zeros(numProds,numProds+1);
                ub=ones(numProds,numProds+1);


                %Make sure states have 0 prob of going back to themselves
                for i=0:numProds-1
                    ub(1+i,2+i)=0;
                end
                
                %MLE fot MC 
                options=optimset('Algorithm','sqp','MaxFunEvals',5000,'Display','off');
                [params,fvalMC] = fmincon(f,horzcat(reshape(lambda,[numProds,1]),P),A,b,[],[],lb,ub,[],options);
                lamMC=params(:,1);
                transP=params(:,2:numProds+1);


                %Simulate over a bunch of revenues
                revTotalMC=0;
                revTotalMNL=0;
                revCountMC=0;
                revCountMNL=0;
                for q=1:numRevs

                    if p==1
                        revListFull = randperm(numProds*10);
                        revList=revListFull(1:numProds);
                        revList(1)=0;
                        revMatrix(q,:)=revList;

                        optAssortMC=MCAssort( params(:,2:numProds+1), revList );
                        optAssortMNL=MNLAssort( revList, prefWeights );
                        revMC = npRev( optAssortMC,customerTypes,  revList );
                        revMNL =  npRev( optAssortMNL,customerTypes,  revList );
                        revTotalMC=revTotalMC + revMC;
                        revTotalMNL=revTotalMNL + revMNL;
                        revOpt=revOpt+NPAssort( customerTypes, revList );
                        if revMNL > revMC
                            revCountMNL = revCountMNL +1;
                        elseif revMNL < revMC 
                            revCountMC = revCountMC +1;
                        end
                    else

                        optAssortMC=MCAssort( params(:,2:numProds+1), revMatrix(q,:) );
                        optAssortMNL=MNLAssort( revMatrix(q,:), prefWeights );
                        revMC = npRev( optAssortMC,customerTypes,  revMatrix(q,:) );
                        revMNL =  npRev( optAssortMNL,customerTypes,  revMatrix(q,:) );
                        revTotalMC=revTotalMC + revMC;
                        revTotalMNL=revTotalMNL + revMNL;
                        if revMNL > revMC
                            revCountMNL = revCountMNL +1;
                        elseif revMNL < revMC 
                            revCountMC = revCountMC +1;
                        end
                    end



                end

                %Write results for revenues and likelihoods
                resultsMatrix(trial,d,1 + 5*(p-1))=MCLikelihoodParams( SVal, ZVal , params, numProds,T/2);
                resultsMatrix(trial,d,2+ 5*(p-1))=revTotalMC;
                resultsMatrix(trial,d,3+ 5*(p-1))=minus_log_LVal(v);
                resultsMatrix(trial,d,4+ 5*(p-1))=revTotalMNL;
                
                %Write results for how many times the MC beat the MNL
                %over the 100 revenue lists
                resultsMatrixRevCounts(trial,d,1 + 2*(p-1)) = revCountMC;
                resultsMatrixRevCounts(trial,d,2 + 2*(p-1)) = revCountMNL;

            end

            resultsMatrix(trial,d,5+ 5*(p-1))=revOpt;

        end
    end
    %Format results to send to csv
    resultsFull=zeros(numDataSets*(numTrials+1),5*(length(trainPercentage)+1));
    rowCount=1;
    for t=1:numTrials
        for d=1:numDataSets 
            for j=1:5
                for p=1:length(trainPercentage)
                    resultsFull(rowCount,j+ 6*(p-1)) = resultsMatrix(t,d,j + 5*(p-1));
                end
            end
            rowCount=rowCount+1;

        end
        rowCount=rowCount+1;

    end

    csvwrite(sprintf('../Results/ResultsStatic%d.csv',numTypes), resultsFull);

    resultsFull=zeros(numDataSets*(numTrials+1),3*(length(trainPercentage)+1));
    rowCount=1;
    for t=1:numTrials
        for d=1:numDataSets 
            for j=1:2
                for p=1:length(trainPercentage)
                    resultsFull(rowCount,j+ 3*(p-1)) = resultsMatrixRevCounts(t,d,j + 2*(p-1));
                end
            end
            rowCount=rowCount+1;

        end
        rowCount=rowCount+1;

    end

    csvwrite(sprintf('../Results/ResultsStaticRevCounts%d.csv', numTypes), resultsFull);
end


%Simulations for censored demand case
resultsMatrix=zeros(numTrials,numDataSets,5*length(trainPercentage));
resultsMatrixRevCounts=zeros(numTrials,numDataSets,2*length(trainPercentage));
for numTypesIndex=1:2
    numTypes=numTypesList(numTypesIndex);
    for trial=1:numTrials
        
        customerTypes = dlmread(sprintf('../Generated_Data/NonParamModel/NonParametric%d-%d.txt',trial,numTypes), '%d');
        for d=1:numDataSets
            
            offer = dlmread(sprintf('../Generated_Data/Sales_Data/SalesDataNoArrival%d-%d-%d.txt',trial,d,numTypes), '%d');
            purchase = dlmread(sprintf('../Generated_Data/Purchases/PurchasesNoArrival%d-%d-%d.txt',trial,d,numTypes), '%d');

            D=size(offer);
            numProds=D(2);

            T=D(1);
            revMatrix=zeros(numRevs,numProds);
            revOpt=0;
            for p=1:length(trainPercentage)

                percent=trainPercentage(p);
                trainCutOff=floor(percent*(T/2));
                S=offer(1:trainCutOff,:);
                Z=purchase(1:trainCutOff,:);
                SVal=offer((T/2)+1:T,:);
                ZVal=purchase((T/2)+1:T,:);

                v_hat = zeros(1,numProds);

                %MNL for no arrival
                f = @(params)LikelihoodMNLNoArrival( S, Z, params, numProds, trainCutOff );
                lb=zeros(1,numProds);
                ub=ones(1,numProds);
                for i=2:numProds
                   lb(1,i)=-Inf;
                   ub(1,i)=Inf;
                end
                [params,fvalMNL] = fmincon(f,v_hat,[],[],[],[],lb,ub,[]);
                v= [0,params(2:numProds)];

                MNLTest =LikelihoodMNLNoArrival( SVal, ZVal, params, numProds, T/2 );
                
                %Exponentiate to get preference weights
                prefWeights= exp(v);

                %Get arrival probs for MC choice model
                lambda = prefWeights/sum(prefWeights);

                %Get transition probs for MC model
                P=zeros(numProds,numProds);

                for i=1:numProds

                    for j=1:numProds

                      if i ~= j
                        P(i,j)= lambda(j)/(1-lambda(i));   
                      end
                   end

                end

                %Create function for MC likelihood
                f = @(params)MCLikelihoodParams( S, Z, params, numProds, trainCutOff );

                %constraint saying that all rows of transition matrix have to sum to less
                %than 1
                Aone=zeros(numProds,numProds^2);
                identity=eye(numProds);

                for i=1:numProds
                    insert=repmat(identity(i,:),1,numProds);
                    Aone(i,:)=insert;
                end

                AoneAdd=zeros(1,numProds^2);

                AoneFinal=vertcat(AoneAdd,Aone);

                Atwo=zeros(numProds+1,numProds);

                Atwo(1,:)=ones(1,numProds);

                A=horzcat(Atwo,AoneFinal);

                b=ones(numProds+1,1);

                lb=zeros(numProds,numProds+1);
                ub=ones(numProds,numProds+1);


                %Make sure states have 0 prob of going back to themselves
                for i=0:numProds-1
                    ub(1+i,2+i)=0;
                end

                options=optimset('Algorithm','sqp','MaxFunEvals',5000,'Display','off');

                [params,fvalMC] = fmincon(f,horzcat(reshape(lambda,[numProds,1]),P),A,b,[],[],lb,ub,[],options);
                lamMC=params(:,1);
                transP=params(:,2:numProds+1);

                %Simulate over a bunch of revenues
                revTotalMC=0;
                revTotalMNL=0;
                revCountMC=0;
                revCountMNL=0;
                for q=1:numRevs

                    if p==1
                        revListFull = randperm(numProds*10);
                        revList=revListFull(1:numProds);
                        revList(1)=0;
                        revMatrix(q,:)=revList;
                        
                        optAssortMC=MCAssort( params(:,2:numProds+1), revList );
                        optAssortMNL=MNLAssort( revList, prefWeights );
                        revMC = npRev( optAssortMC,customerTypes,  revList );
                        revMNL =  npRev( optAssortMNL,customerTypes,  revList );
                        revTotalMC=revTotalMC + revMC;
                        revTotalMNL=revTotalMNL + revMNL;
                        revOpt=revOpt+NPAssort( customerTypes, revList );
                        if revMNL > revMC
                            revCountMNL = revCountMNL +1;
                        elseif revMNL < revMC 
                            revCountMC = revCountMC +1;
                        end
                    else

                        optAssortMC=MCAssort( params(:,2:numProds+1), revMatrix(q,:) );
                        optAssortMNL=MNLAssort( revMatrix(q,:), prefWeights );
                        revMC = npRev( optAssortMC,customerTypes,  revMatrix(q,:) );
                        revMNL =  npRev( optAssortMNL,customerTypes,  revMatrix(q,:) );
                        revTotalMC=revTotalMC + revMC;
                        revTotalMNL=revTotalMNL + revMNL;
                        if revMNL > revMC
                            revCountMNL = revCountMNL +1;
                        elseif revMNL < revMC 
                            revCountMC = revCountMC +1;
                        end
                    end



                end


                resultsMatrix(trial,d,1 + 5*(p-1))=MCLikelihoodParams( SVal, ZVal , params, numProds,T/2);
                resultsMatrix(trial,d,2+ 5*(p-1))=revTotalMC;
                resultsMatrix(trial,d,3+ 5*(p-1))=MNLTest;
                resultsMatrix(trial,d,4+ 5*(p-1))=revTotalMNL;

                resultsMatrixRevCounts(trial,d,1 + 2*(p-1)) = revCountMC;
                resultsMatrixRevCounts(trial,d,2 + 2*(p-1)) = revCountMNL;

            end

            resultsMatrix(trial,d,5+ 5*(p-1))=revOpt;

        end
    end
    resultsFull=zeros(numDataSets*(numTrials+1),5*(length(trainPercentage)+1));
    rowCount=1;
    for t=1:numTrials
        for d=1:numDataSets 
            for j=1:5
                for p=1:length(trainPercentage)
                    resultsFull(rowCount,j+ 6*(p-1)) = resultsMatrix(t,d,j + 5*(p-1));
                end
            end
            rowCount=rowCount+1;

        end
        rowCount=rowCount+1;

    end

    csvwrite(sprintf('../Results/ResultsStaticNoArrival%d.csv',numTypes), resultsFull);

    resultsFull=zeros(numDataSets*(numTrials+1),3*(length(trainPercentage)+1));
    rowCount=1;
    for t=1:numTrials
        for d=1:numDataSets 
            for j=1:2
                for p=1:length(trainPercentage)
                    resultsFull(rowCount,j+ 3*(p-1)) = resultsMatrixRevCounts(t,d,j + 2*(p-1));
                end
            end
            rowCount=rowCount+1;

        end
        rowCount=rowCount+1;

    end

    csvwrite(sprintf('../Results/ResultsStaticRevCountsNoArrival%d.csv', numTypes), resultsFull);
end


%Simulation for moments method
numTypes = 50;
trainPercentage=[0.4,1.0];
resultsMatrix=zeros(numTrials,numDataSets,10*length(trainPercentage));

for trial=1:numTrials
    customerTypes = dlmread(sprintf('../Generated_Data/NonParamModel/NonParametric%d-%d.txt',trial,numTypes), '%d');
    for d=1:numDataSets
        
        offer = dlmread(sprintf('../Generated_Data/Sales_Data/SalesDataNested%d-%d-%d.txt',trial,d,numTypes), '%d');
        purchase = dlmread(sprintf('../Generated_Data/Purchases/PurchasesNested%d-%d-%d.txt',trial,d,numTypes), '%d');
        
        D=size(offer);
        numProds=D(2);
        T=D(1);
        revMatrix=zeros(numRevs,numProds);
        revOpt=0;
        for p=1:length(trainPercentage)

            percent=trainPercentage(p);
            trainCutOff=floor(percent*(T/2));
            S=offer(1:trainCutOff,:);
            Z=purchase(1:trainCutOff,:);
            SVal=offer((T/2)+1:T,:);
            ZVal=purchase((T/2)+1:T,:);
            
            [offerMatrix,countsMatrix, countDiffAssorts] = EmpiricalPurchaseProbs(percent,numProds, offer, purchase);

            v_hat = zeros(1,numProds);
            %Create and maximize log likelihood for MNL
            minus_log_L = @(v) -( sum(sum(Z.*repmat([0 v(2:numProds)], trainCutOff,1),2))  - (sum(log(sum(exp(repmat([0 v(2:numProds)], trainCutOff,1)).*S,2)))) );
            minus_log_LVal = @(v) -( sum(sum(ZVal.*repmat([0 v(2:numProds)],T/2,1),2))  - (sum(log(sum(exp(repmat([0 v(2:numProds)],T/2,1)).*SVal,2)))) );
            [v,fvalMNL] = fmincon(minus_log_L,v_hat,[],[]);


            %Exponentiate to get preference weights
            prefWeights= exp(v);

            %Get arrival probs for MC choice model
            lambda = prefWeights/sum(prefWeights);

            %Get transition probs for MC model
            P=zeros(numProds,numProds);

            for i=1:numProds

                for j=1:numProds

                  if i ~= j
                    P(i,j)= lambda(j)/(1-lambda(i));   
                  end
               end

            end

            %Create function for MC likelihood
            f = @(params)SquaredError(params,offerMatrix,countsMatrix, countDiffAssorts,numProds );

            %constraint saying that all rows of transition matrix have to sum to less
            %than 1
            Aone=zeros(numProds,numProds^2);
            identity=eye(numProds);

            for i=1:numProds
                insert=repmat(identity(i,:),1,numProds);
                Aone(i,:)=insert;
            end

            AoneAdd=zeros(1,numProds^2);

            AoneFinal=vertcat(AoneAdd,Aone);

            Atwo=zeros(numProds+1,numProds);

            Atwo(1,:)=ones(1,numProds);

            A=horzcat(Atwo,AoneFinal);

            b=ones(numProds+1,1);

            lb=zeros(numProds,numProds+1);
            ub=ones(numProds,numProds+1);


            %Make sure states have 0 prob of going back to themselves
            for i=0:numProds-1
                ub(1+i,2+i)=0;
            end

            options=optimset('Algorithm','sqp','MaxFunEvals',5000,'Display','off');

            [params,fvalMC] = fmincon(f,horzcat(reshape(lambda,[numProds,1]),P),A,b,[],[],lb,ub,[],options);
            
                        %Create function for MC likelihood
            fLike = @(params)MCLikelihoodParams( S, Z, params, numProds, trainCutOff );

            [paramsMLE,fvalMCMLE] = fmincon(fLike,horzcat(reshape(lambda,[numProds,1]),P),A,b,[],[],lb,ub,[],options);
            
            %Check true squared error
            [MNLError, MCError] = TrueSquaredError(params,prefWeights,customerTypes, offerMatrix,countDiffAssorts,numProds, numTypes );
            

            



            %Simulate over a bunch of revenues
            revTotalMC=0;
            revTotalMNL=0;
            revCountMC=0;
            revCountMNL=0;
            revTotalMCMLE=0;
            for q=1:numRevs

                if p==1
                    revListFull = randperm(numProds*10);
                    revList=revListFull(1:numProds);
                    revList(1)=0;
                    revMatrix(q,:)=revList;

                    optAssortMC=MCAssort( params(:,2:numProds+1), revList );
                    optAssortMCMLE=MCAssort( paramsMLE(:,2:numProds+1), revList );
                    optAssortMNL=MNLAssort( revList, prefWeights );
                    revMC = npRev( optAssortMC,customerTypes,  revList );
                    revMCMLE = npRev( optAssortMCMLE,customerTypes,  revList );
                    revMNL =  npRev( optAssortMNL,customerTypes,  revList );
                    revTotalMC=revTotalMC + revMC;
                    revTotalMCMLE=revTotalMCMLE + revMCMLE;
                    revTotalMNL=revTotalMNL + revMNL;
                    revOpt=revOpt+NPAssort( customerTypes, revList );
                    if revMNL > revMC
                        revCountMNL = revCountMNL +1;
                    elseif revMNL < revMC
                        revCountMC = revCountMC +1;
                    end
                else
                    optAssortMC=MCAssort( params(:,2:numProds+1), revMatrix(q,:) );
                    optAssortMCMLE=MCAssort( paramsMLE(:,2:numProds+1), revMatrix(q,:) );
                    optAssortMNL=MNLAssort( revMatrix(q,:), prefWeights );
                    revMC = npRev( optAssortMC,customerTypes,  revMatrix(q,:) );
                    revMCMLE = npRev( optAssortMCMLE,customerTypes,  revMatrix(q,:) );
                    revMNL =  npRev( optAssortMNL,customerTypes,  revMatrix(q,:) );
                    revTotalMC=revTotalMC + revMC;
                    revTotalMCMLE=revTotalMCMLE + revMCMLE;
                    revTotalMNL=revTotalMNL + revMNL;
                    revOpt=revOpt+NPAssort( customerTypes, revMatrix(q,:) );
                    if revMNL > revMC
                        revCountMNL = revCountMNL +1;
                    elseif revMNL < revMC
                        revCountMC = revCountMC +1;
                    end
                end



            end
            
            resultsMatrix(trial,d,1 + 10*(p-1))=MCLikelihoodParams( SVal, ZVal , params, numProds,T/2);
            resultsMatrix(trial,d,2+ 10*(p-1))=minus_log_LVal(v);
            resultsMatrix(trial,d,3+ 10*(p-1)) = MCError;
            resultsMatrix(trial,d,4+ 10*(p-1)) = MNLError;
            resultsMatrix(trial,d,5+ 10*(p-1))=revTotalMC;
            resultsMatrix(trial,d,6+ 10*(p-1))=revTotalMNL;
            resultsMatrix(trial,d,7 + 10*(p-1)) = revCountMC;
            resultsMatrix(trial,d,8 + 10*(p-1)) = revCountMNL;
            resultsMatrix(trial,d,9 + 10*(p-1))=MCLikelihoodParams( SVal, ZVal , paramsMLE, numProds,T/2);
            resultsMatrix(trial,d,10+ 10*(p-1))=revTotalMCMLE;
          

        end

 
    
    end
end
resultsFull=zeros(numDataSets*(numTrials+1),4*(length(trainPercentage)+1));
rowCount=1;
for t=1:numTrials
    for d=1:numDataSets 
        for j=1:10
            for p=1:length(trainPercentage)
                resultsFull(rowCount,j+ 11*(p-1)) = resultsMatrix(t,d,j + 10*(p-1));
            end
        end
        rowCount=rowCount+1;
        
    end
    rowCount=rowCount+1;
   
end

csvwrite(sprintf('../Results/ResultsStaticMomentsNested%d.csv',numTypes), resultsFull);

